@@ -41,4 +41,9 @@ extern void SYS_Linux_SetScheduler(int SchedPriority);
 
 extern int SYS_Linux_CheckKernelVersion(int req_major, int req_minor);
 
+extern int SYS_Linux_OpenPHC(const char *path, int phc_index);
+
+extern int SYS_Linux_GetPHCSample(int fd, int nocrossts, double precision, int *reading_mode,
+                                  struct timespec *phc_ts, struct timespec *sys_ts, double *err);
+
 #endif  /* GOT_SYS_LINUX_H */
